<?php

/**
 * Class Dmarcian_Admin_Actions implements WP Ajax actions to interact with the cached data.
 */
class Dmarcian_Admin_Actions extends Dmarcian_Actions
{
    /**
     * @const NONCE_ACTION Key for nonce generation and verification.
     */
    const NONCE_ACTION = 'dmarcian-admin';

    /**
     * @const API_SCRIPT_HANDLE Handler name of the included API script.
     */
    const API_SCRIPT_HANDLE = 'dmarcian-admin-api';

    /**
     * @var Dmarcian_API API wrapper.
     */
    protected $api;

    /**
     * @var Dmarcian_Cache Database access.
     */
    protected $cache;

    /**
     * @var array List of all supported domains.
     */
    protected $domains = array();

    /**
     * @var float Current plugin version to add to scripts.
     */
    protected $version;

    /**
     * Verifies that the API is present.
     */
    protected function check_api()
    {
        if ( ! $this->api ) {
            $this->return_status( 500 );
        }
    }

    /**
     * Verifies that the cache is present.
     */
    protected function check_cache()
    {
        if ( ! $this->cache ) {
            $this->return_status( 500 );
        }
    }

    /**
     * Dmarcian_Admin_Actions constructor. Hooks all necessary actions to expose endpoints.
     *
     * @param array $params MUST have the api key set. Otherwise all requests will result in 500.
     *                      MUST have the cache key set. Otherwise all requests will result in 500.
     *                      MUST have the capability key set. Otherwise the API will be ALWAYS forbidden.
     *                      MUST have the domains key set. Otherwise refresh cache will do nothing.
     *                      MAY have the version key set. Otherwise all assets will be versioned as the current WP version.
     */
    protected function __construct( $params = array() )
    {
        parent::__construct( $params );

        // init params
        if ( array_key_exists( 'api', $params ) ) {
            $this->api = $params['api'];
        }
        if ( array_key_exists( 'cache', $params ) ) {
            $this->cache = $params['cache'];
        }
        if ( array_key_exists( 'domains', $params ) ) {
            $this->domains = $params['domains'];
        }
        if ( array_key_exists( 'version', $params ) ) {
            $this->version = $params['version'];
        }

        // load javascript API
        add_action( 'admin_enqueue_scripts', array( $this, 'add_scripts' ) );

        // load action handlers
        add_action( 'wp_ajax_dmarcian_admin_check_dmarc', array( $this, 'check_dmarc' ) );
        add_action( 'wp_ajax_dmarcian_admin_check_dkim', array( $this, 'check_dkim' ) );
        add_action( 'wp_ajax_dmarcian_admin_check_spf', array( $this, 'check_spf' ) );
        add_action( 'wp_ajax_dmarcian_admin_delete_dkim', array( $this, 'delete_dkim' ) );
        add_action( 'wp_ajax_dmarcian_admin_refresh_cache', array( $this, 'refresh_cache' ) );
        add_action( 'wp_ajax_dmarcian_admin_refresh_cache_progress', array( $this, 'refresh_cache_progress' ) );
        add_action( 'wp_ajax_dmarcian_admin_get_global_state', array( $this, 'get_global_state' ) );
    }

    /**
     * Enqueues the javascript tools for calling the API.
     */
    public function add_scripts()
    {
        wp_enqueue_script(
            self::API_SCRIPT_HANDLE,
            plugins_url( '/js/api.js', __FILE__ ),
            array( 'jquery' ),
            $this->version
        );
        wp_localize_script(
            self::API_SCRIPT_HANDLE,
            'dmarcianAPIParams',
            array(
                'ajaxUrl' => admin_url( 'admin-ajax.php' ),
                'nonce' => wp_create_nonce( self::NONCE_ACTION ),
                'nonceKey' => self::NONCE_KEY
            )
        );
    }

    /**
     * Creates the array returned as JSON for a Dmarcian_State object.
     *
     * @param Dmarcian_State $state
     * @return array
     */
    protected function create_state_response( $state )
    {
        $response = array( 'state' => $state->get_state() );
        if ( $state->is_error() ) {
            $response['error'] = $state->get_error_string();
        }

        return $response;
    }

    /**
     * DMARC inspect proxy action.
     */
    public function check_dmarc()
    {
        $this->check_access();
        $this->check_api();

        $state = $this->api->check_dmarc( $_POST['domain'] );

        wp_send_json( $this->create_state_response( $state ) );
    }

    /**
     * SPF inspect proxy action.
     */
    public function check_spf()
    {
        $this->check_access();
        $this->check_api();

        $state = $this->api->check_spf( $_POST['domain'] );

        wp_send_json( $this->create_state_response( $state ) );
    }

    /**
     * DKIM inspect proxy action.
     */
    public function check_dkim()
    {
        $this->check_access();
        $this->check_api();

        $state = $this->api->check_dkim( $_POST['domain'], $_POST['selector'] );

        wp_send_json( $this->create_state_response( $state ) );
    }

    /**
     * Delete DKIM action.
     */
    public function delete_dkim()
    {
        $this->check_access();
        $this->check_cache();

        $this->cache->delete_dkim_state( $_POST['domain'], $_POST['selector'] );

        wp_send_json( array() );
    }

    /**
     * Refresh the whole cache action.
     */
    public function refresh_cache()
    {
        $this->check_access();
        $this->check_api();

        // set we're fresh to prevent concurrency
        Dmarcian_Config::set_cache_is_fresh();

        // mark that we've started a refresh
        Dmarcian_Config::set_cache_refresh_progress( 0, 100 );

        // count how many checks we have to make
        $total = 0;
        foreach ( $this->domains as $domain => $states ) {
            // dmarc and spf
            $total += 2;

            // dkim
            $total += count( $states['dkim'] );
        }

        // start the slow process
        $progress = 0;
        foreach ( $this->domains as $domain => $states ) {
            // dmarc
            $this->api->check_dmarc( $domain );
            Dmarcian_Config::set_cache_refresh_progress( ++$progress, $total );

            // spf
            $this->api->check_spf( $domain );
            Dmarcian_Config::set_cache_refresh_progress( ++$progress, $total );

            // check all selectors
            foreach ( $states['dkim'] as $selector => $state ) {
                $this->api->check_dkim( $domain, $selector );
                Dmarcian_Config::set_cache_refresh_progress( ++$progress, $total );
            }
        }

        // mark that the refresh has ended
        Dmarcian_Config::delete_cache_refresh_progress();

        wp_send_json( array() );
    }

    /**
     * Returns the progress of the refresh cache.
     */
    public function refresh_cache_progress()
    {
        $this->check_access();

        wp_send_json( array( 'progress' => Dmarcian_Config::get_cache_refresh_progress() ) );
    }

    /**
     * Returns the global state of all policies like green or red.
     */
    public function get_global_state()
    {
        $this->check_access();
        $this->check_cache();

        wp_send_json( array( 'valid' => $this->cache->is_all_valid( $this->domains ) ) );
    }
}