<?php

/**
 * Class Dmarcian_API provides an interface to the external dmarcian API.
 */
class Dmarcian_API extends Dmarcian_Singleton
{
    /**
     * @const BASE_URL The base url of the dmarcian API.
     */
    const BASE_URL = 'https://wp-tools.dmarcian.com';

    /**
     * @const LEVEL_ERROR Error level of the messages returned by the dmarcian API endpoints.
     */
    const LEVEL_ERROR = 'err';

    /**
     * @var Dmarcian_Cache Access to the cache.
     */
    protected $cache;

    /**
     * Makes a call to the dmarcian API.
     *
     * @param string $url  The relative url to call.
     * @param array  $data Data to POST.
     * @return Dmarcian_State
     */
    protected function call( $url, $data = array() )
    {
        $state = new Dmarcian_State();

        // fetch data
        $response = wp_remote_post(
            sprintf( '%s%s', self::BASE_URL, $url ),
            array( 'body' => $data )
        );

        // handle communication problems
        if ( $response instanceof WP_Error ) {
            return $state->error( array( 'wp_error' => serialize( $response ) ) );
        }
        if ( ! is_array( $response ) ) {
            return $state->error( array( 'empty' => null ) );
        }
        if ( $response['response']['code'] != 200 ) {
            return $state->error( array( 'api_error' => json_decode( $response['body'], true ) ) );
        }

        // all good - set the response data
        $state->set_data( json_decode( $response['body'], true ) );

        return $state;
    }

    /**
     * Checks a list of dmarcian response messages for present errors.
     *
     * @param array $messages
     * @return bool
     */
    protected function have_errors( $messages )
    {
        foreach ( $messages as $message ) {
            if ( $message['level'] == self::LEVEL_ERROR ) {
                return true;
            }
        }

        return false;
    }

    /**
     * Dmarcian_API constructor. Sets necessary dependencies.
     *
     * @param array $params MUST have the cache key set. Otherwise no results will be cached.
     */
    protected function __construct( $params = array() )
    {
        if ( array_key_exists( 'cache', $params ) ) {
            $this->cache = $params['cache'];
        }
    }

    /**
     * DMARC inspect proxy action.
     *
     * @param string $domain
     * @return Dmarcian_State
     */
    public function check_dmarc( $domain )
    {
        $state = $this->call( '/api/dmarc/inspect/', array(
            'domain' => $domain
        ));

        // check for invalid
        if ( ! $state->is_error() ) {
            $data = $state->get_data();

            if ( $this->have_errors( $data['messages'] ) ) {
                $state->invalid();
            }
            if ( count( $data['records'] ) && $this->have_errors( $data['records'][0]['messages'] ) ) {
                $state->invalid();
            }
        }

        // cache the state
        if ( $this->cache ) {
            $this->cache->set_dmarc_state( $domain, $state->get_state() );
        }

        return $state;
    }

    /**
     * SPF inspect proxy action.
     *
     * @param string $domain
     * @return Dmarcian_State
     */
    public function check_spf( $domain )
    {
        $state = $this->call( '/api/spf/inspect/', array(
            'domain' => $domain
        ));

        // check for invalid
        if ( ! $state->is_error() ) {
            $data = $state->get_data();

            if ( ! count( $data['records'] ) ) {
                $state->invalid();
            }
            if ( $this->have_errors( $data['messages'] ) ) {
                $state->invalid();
            }
        }

        // cache the state
        if ( $this->cache ) {
            $this->cache->set_spf_state( $domain, $state->get_state() );
        }

        return $state;
    }

    /**
     * DKIM inspect proxy action.
     *
     * @param string $domain
     * @param string $selector
     * @return Dmarcian_State
     */
    public function check_dkim( $domain, $selector )
    {
        $state = $this->call( '/api/dkim/inspect/', array(
            'domain' => $domain,
            'selector' => $selector
        ));

        // check for invalid
        if ( ! $state->is_error() && ! $state->get_data()['valid'] ) {
            $state->invalid();
        }

        // cache the state
        if ( $this->cache ) {
            $this->cache->set_dkim_state( $domain, $selector, $state->get_state() );
        }

        return $state;
    }
}