<?php

/**
 * Class Dmarcian_Cache provides an interface for managing the cached data.
 */
class Dmarcian_Cache extends Dmarcian_Singleton
{
    /**
     * @const TABLE_NAME Name of the database table.
     */
    const TABLE_NAME = 'dmarcian_cache';

    /**
     * @const POLICY_DMARC DMARC policy.
     */
    const POLICY_DMARC = 'dmarc';

    /**
     * @const POLICY_SPF SPF policy.
     */
    const POLICY_SPF = 'spf';

    /**
     * @const POLICY_DKIM DKIM policy.
     */
    const POLICY_DKIM = 'dkim';

    /**
     * @const STATE_VALID When the policy is valid.
     */
    const STATE_VALID = 'valid';

    /**
     * @const STATE_INVALID When the policy is invalid.
     */
    const STATE_INVALID = 'invalid';

    /**
     * @const STATE_ERROR When the policy checking encountered problems.
     */
    const STATE_ERROR = 'error';

    /**
     * @var wpdb
     */
    protected $db;

    /**
     * @var null|array
     */
    protected $cache;

    /**
     * Builds the full table name.
     *
     * @return string
     */
    public static function get_table_name()
    {
        global $wpdb;
        return $wpdb->prefix . self::TABLE_NAME;
    }

    /**
     * Dmarcian_Cache constructor loads the dependencies.
     *
     * @param array $params
     */
    protected function __construct( $params = array() )
    {
        global $wpdb;
        $this->db = $wpdb;
    }

    /**
     * Generates the unique identifier for a row in the cache table.
     *
     * @param string $domain
     * @param string $selector
     * @param string $policy
     * @return string
     */
    protected function get_id( $domain, $selector, $policy )
    {
        return md5( sprintf( '%s|%s|%s', $domain, $selector, $policy ) );
    }

    /**
     * Retrives all cached data from the database.
     *
     * @return array
     */
    protected function get_cache()
    {
        if ( ! $this->cache ) {
            $table_name = self::get_table_name();

            $sql = "SELECT domain, selector, policy, state
                FROM $table_name
                ORDER BY domain ASC, selector ASC;";

            $this->cache = $this->db->get_results( $sql, ARRAY_A );
        }

        return $this->cache;
    }

    /**
     * Caches the DMARC state of a domain.
     *
     * @param string $domain
     * @param string $state
     */
    public function set_dmarc_state( $domain, $state )
    {
        $id = $this->get_id( $domain, '', self::POLICY_DMARC );

        $this->db->replace( self::get_table_name(), array(
            'id' => $id,
            'domain' => $domain,
            'selector' => '',
            'policy' => self::POLICY_DMARC,
            'state' => $state
        ));
    }

    /**
     * Caches the SPF state of a domain.
     *
     * @param string $domain
     * @param string $state
     */
    public function set_spf_state( $domain, $state )
    {
        $id = $this->get_id( $domain, '', self::POLICY_SPF );

        $this->db->replace( self::get_table_name(), array(
            'id' => $id,
            'domain' => $domain,
            'selector' => '',
            'policy' => self::POLICY_SPF,
            'state' => $state
        ));
    }

    /**
     * Caches the DKIM state of a domain - selector pair.
     *
     * @param string $domain
     * @param string $selector
     * @param string $state
     */
    public function set_dkim_state( $domain, $selector, $state )
    {
        $id = $this->get_id( $domain, $selector, self::POLICY_DKIM );

        $this->db->replace( self::get_table_name(), array(
            'id' => $id,
            'domain' => $domain,
            'selector' => $selector,
            'policy' => self::POLICY_DKIM,
            'state' => $state
        ));
    }

    /**
     * Deletes a DKIM state of a domain - selector pair.
     *
     * @param string $domain
     * @param string $selector
     */
    public function delete_dkim_state( $domain, $selector )
    {
        if ( ! $domain || ! $selector ) {
            return;
        }

        $this->db->delete( self::get_table_name(), array(
            'domain' => $domain,
            'selector' => $selector,
            'policy' => self::POLICY_DKIM
        ));
    }

    /**
     * Create a state node structure for certain domain.
     *
     * @return array
     */
    protected function init_state_node()
    {
        return array(
            'dmarc' => null,
            'spf' => null,
            'dkim' => array()
        );
    }

    /**
     * Builds a convenient data tree with all cached entries.
     *
     * @param array $domains Domains to build the tree for.
     * @return array
     */
    public function get_state_tree( $domains = array() )
    {
        $cache = $this->get_cache();

        // prepare the tree structure
        $tree = array();
        $domains = array_unique( $domains );
        foreach ( $domains as $domain ) {
            $tree[$domain] = $this->init_state_node();
        }

        // build the tree with all existing data
        foreach ( $cache as $row ) {
            $domain = $row['domain'];
            $selector = $row['selector'];

            // skip if the domain is not requested
            if ( ! array_key_exists( $domain, $tree ) ) {
                continue;
            }

            // update node
            switch ( $row['policy'] ) {
                case self::POLICY_DMARC:
                    $tree[$domain]['dmarc'] = $row['state'];
                    break;
                case self::POLICY_SPF:
                    $tree[$domain]['spf'] = $row['state'];
                    break;
                case self::POLICY_DKIM:
                    $tree[$domain]['dkim'][$selector] = $row['state'];
                    break;
            }
        }

        // sort by domain
        ksort($tree);

        return $tree;
    }

    /**
     * Checks all states in a state tree to verify that everything is valid.
     *
     * @param array $state_tree The state tree.
     * @return bool
     */
    public function is_all_valid( $state_tree = array() )
    {
        foreach ( $state_tree as $domain => $states ) {
            if ( $states['dmarc'] != self::STATE_VALID ) {
                return false;
            }

            if ( $states['spf'] != self::STATE_VALID ) {
                return false;
            }

            foreach ( $states['dkim'] as $selector => $state ) {
                if ( $state != self::STATE_VALID ) {
                    return false;
                }
            }
        }

        return true;
    }
}