<?php
/*
Plugin Name: dmarcian
Plugin URI:  https://wordpress.org/plugins/dmarcian
Description: Inspect the state of your DMARC, DKIM and SPF policies.
Version:     1.0
Author:      dmarcian.com
Author URI:  https://dmarcian.com
*/

// if this file is called directly, abort
if ( ! defined( 'WPINC' ) ) {
    die;
}

// load base classes
require_once 'base/singleton.php';
require_once 'base/actions.php';

// load options
require_once 'config.php';

// load data access
require_once 'cache.php';
require_once 'migrations.php';

// load plugin API classes
require_once 'admin/actions.php';
require_once 'admin/state.php';
require_once 'admin/api.php';

/**
 * Class Dmarcian - main class of the plugin.
 */
class Dmarcian extends Dmarcian_Singleton
{
    /**
     * @const REQUIRED_CAPABILITY What capability the user should have to access the plugin.
     */
    const REQUIRED_CAPABILITY = 'manage_options';

    /**
     * @const VERSION Current plugin version.
     */
    const VERSION = 1.0;

    /**
     * @var array A list of website domains as a tree with states.
     */
    private $domains = array();

    /**
     * @var Dmarcian_API The bridge to dmarcian.
     */
    private $api;

    /**
     * @var Dmarcian_Admin_Actions Actions exposing the admin API.
     */
    private $admin_actions;

    /**
     * @var Dmarcian_Cache Access to the database.
     */
    private $cache;

    /**
     * @var bool
     */
    private $multisite;

    /**
     * Dmarcian constructor - hooks into admin.
     *
     * @param array $params
     */
    protected function __construct( $params = array() )
    {
        if ( ! is_admin() || ! current_user_can( self::REQUIRED_CAPABILITY ) ) {
            return;
        }

        $this->multisite = is_multisite();

        // add hooks
        add_action( 'admin_menu', array( $this, 'add_menu' ) );
        add_action( 'admin_enqueue_scripts', array( $this, 'add_scripts' ) );

        // stop here if we're multisite - we can't do much about it right now...
        if ( $this->multisite ) {
            return;
        }

        // update the database to latest version
        Dmarcian_Migrations::update();

        // load the cache
        $this->cache = Dmarcian_Cache::get_instance();

        // load domains
        $this->load_domains();

        // load the dmarcian API
        $this->api = Dmarcian_API::get_instance( array(
            'cache' => $this->cache
        ));

        // load all actions
        $this->admin_actions = Dmarcian_Admin_Actions::get_instance( array(
            'api' => $this->api,
            'cache' => $this->cache,
            'domains' => $this->domains,
            'capability' => self::REQUIRED_CAPABILITY,
            'version' => self::VERSION
        ));
    }

    /**
     * Finds all available domains in the current installation.
     */
    protected function load_domains()
    {
        $parts = parse_url( get_site_url() );
        $domains = ( $parts ) ? array( str_replace( 'www.', '', $parts['host'] ) ) : array();
        $this->domains = $this->cache->get_state_tree( $domains );
    }

    /**
     * Adds a top-level menu.
     */
    public function add_menu()
    {
        add_menu_page(
            'dmarcian',
            'dmarcian<span class="dmarcian-refresh-progress" style="display:none"></span>',
            self::REQUIRED_CAPABILITY,
            'dmarcian',
            array( $this, 'render_dashboard' ),
            'none'
        );
    }

    /**
     * Renders the dashboard.
     */
    public function render_dashboard()
    {
        // we need some variables in our partial
        $domains = $this->domains;
        $multisite = $this->multisite;
        include( 'admin/partials/dashboard.php' );
    }

    /**
     * Loads all scripts needed in the admin.
     *
     * @param string $hook
     */
    public function add_scripts( $hook )
    {
        // include the dashboard only if we need it
        if ( ! $this->multisite && $hook == 'toplevel_page_dmarcian' ) {
            wp_enqueue_script(
                'dmarcian-admin-dashboard',
                plugin_dir_url( __FILE__ ) . 'admin/js/dashboard.js',
                array( 'jquery', 'dmarcian-admin-api' ),
                self::VERSION,
                false
            );
        }

        // always load menu
        $menu_dependencies = array( 'jquery' );
        if ( ! $this->multisite ) {
            $menu_dependencies[] = 'dmarcian-admin-api';
        }
        wp_enqueue_script(
            'dmarcian-admin-menu',
            plugin_dir_url( __FILE__ ) . 'admin/js/menu.js',
            $menu_dependencies,
            self::VERSION,
            false
        );
        wp_localize_script(
            'dmarcian-admin-menu',
            'dmarcianParams',
            array(
                'multisite' => $this->multisite,
                'refreshCache' => Dmarcian_Config::is_cache_old(),
                'refreshInProgress' => Dmarcian_Config::is_cache_refresh_in_progress()
            )
        );
        wp_enqueue_style(
            'dmarcian-admin-menu',
            plugin_dir_url( __FILE__ ) . 'admin/css/menu.css'
        );
    }
}

// run
add_action( 'plugins_loaded', array( 'Dmarcian', 'get_instance' ) );